// File: FW_TangentPlayer.ino
// 2015 jan@breem.nl
//
//  http://86.91.251.42/JanBeeProjects/TFT_Touch2p8inch/TFT-Touch8p2.htm 
//
// This file implements a Graphical User Interface (GUI) for a Tangential Vinyl Record Player.
// It is not finished yet, but it demonstrates the use of some of the controls.
// It is designed for an Arduino MEGA2560, but it will run on many other Arduinos too.
// 
// The serial output is just for diagnostic purposes. Omit it when you dont need it anymore.
//

#include <SPI.h>
#include <SeeedTouchScreen.h> 
#include "TFTv2.h"
#include "LibTFT2p8.h"
#include "controls.h"

enum t_State {stIdle, stPlaying, stPaused, stStopping}; 
t_State State;
t_RecordType RecordType;
bool FirstTime;

//******************************************************************************************************
void ReDrawButtons (void)
{
  switch (State)
  {
  case stIdle:  
    DrawButtPlay (20, 10, GREEN, TouchPlay, FirstTime);
    DrawButtPause (90, 10, GRAY1, TouchPause, FirstTime);
    DrawButtStop (160, 10, GRAY1, TouchStop, FirstTime);
    DrawButtRecordType (90, 80, RecordType, true, TouchRecordType, FirstTime);
    DrawButtReverse (160, 80, GRAY1, TouchReverse, FirstTime);
    DrawButtForward (20, 80, GRAY1, TouchForward, FirstTime);
    break;
  case stPlaying:
    DrawButtPlay (20, 10, GRAY1, TouchPlay, FirstTime);
    DrawButtPause (90, 10, CYAN, TouchPause, FirstTime);
    DrawButtStop (160, 10, GREEN, TouchStop, FirstTime);
    DrawButtRecordType (90, 80, RecordType, true, TouchRecordType, FirstTime);
    DrawButtReverse (160, 80, GRAY1, TouchReverse, FirstTime);
    DrawButtForward (20, 80, GRAY1, TouchForward, FirstTime);
    break;
  case stPaused:
    DrawButtPlay (20, 10, GREEN, TouchPlay, FirstTime);
    DrawButtPause (90, 10, GREEN, TouchPause, FirstTime);
    DrawButtStop (160, 10, GREEN, TouchStop, FirstTime);
    DrawButtRecordType (90, 80, RecordType, false, TouchRecordType, FirstTime);
    DrawButtReverse (160, 80, CYAN, TouchReverse, FirstTime);
    DrawButtForward (20, 80, CYAN, TouchForward, FirstTime);
    break;
  case stStopping:  
    DrawButtPlay (20, 10, GRAY1, TouchPlay, FirstTime);
    DrawButtPause (90, 10, GRAY1, TouchPause, FirstTime);
    DrawButtStop (160, 10, GRAY1, TouchStop, FirstTime);
    DrawButtRecordType (90, 80, RecordType, true, TouchRecordType, FirstTime);
    DrawButtReverse (160, 80, GRAY1, TouchReverse, FirstTime);
    DrawButtForward (20, 80, GRAY1, TouchForward, FirstTime);
    break;
  }
  if (RecordType == LP) tft_DrawString ("LP 33", 20, 190, 5, RED, BLACK);
  if (RecordType == EP) tft_DrawString ("EP 33", 20, 190, 5, RED, BLACK);
  if (RecordType == SI) tft_DrawString ("SI 45", 20, 190, 5, RED, BLACK);
  FirstTime = false; // prevent multiple entries in TouchList
}

//******************************************************************************************************
void TouchPlay (void)
{
  Serial.println("Play Pressed: ");
  State = stPlaying;
  ReDrawButtons();
}

//******************************************************************************************************
void TouchPause (void)
{
  Serial.println("Pause Pressed: ");
  if (State == stStopping) return;
  if (State == stPlaying) State = stPaused;
  else if (State == stPaused) State = stPlaying;
  ReDrawButtons();
}

//******************************************************************************************************
void TouchStop (void)
{
  Serial.println("Stop Pressed: ");
  if (State == stStopping) return;
  State = stStopping;
  ReDrawButtons();
  delay (2000);   // 2 seconds
  State = stIdle;
  ReDrawButtons();
}

//******************************************************************************************************
void TouchForward (void)
{
  Serial.println("Forward Pressed: ");
  ReDrawButtons();
}

//******************************************************************************************************
void TouchRecordType (void)
{
  Serial.println("RecordType Pressed: ");
  if ((State == stPlaying) || (State == stStopping)) return;
  switch (RecordType)
  {
    case LP: RecordType = EP; break;
    case EP: RecordType = SI; break;
    case SI: RecordType = LP; break;
  }
  Serial.print("RecordType is now ");
  Serial.println(RecordType);
  ReDrawButtons();
}

//******************************************************************************************************
void TouchReverse (void)
{
  Serial.println("Reverse Pressed: ");
  ReDrawButtons();
}

//******************************************************************************************************
void setup()
{
  Serial.begin(9600); 
  tft_init_LibTFT2p8();
  InitVUmeter();
  RecordType = LP;
  FirstTime = true; // allows callback function pointers to be entered into the TouchList
  ReDrawButtons();
}

//******************************************************************************************************
void loop()
{
  int dummy = tft_DetectTouch(); // this calls the proper callbackfunction 
  delay(40); // prevent calling tft_DetectTouch to often
  static byte n;
  UpdateVUmeter (n*10, n*9); 
  if (n++ > 22) {n = 0;}
}

