// File: LibTFT2p8.cpp
// Library for 2.8 inch TFT Resistive Touch Screen
// Allows Flip and Rotate
// 2015-09 jan@breem.nl

#include "TFTv2.h"
#include <SeeedTouchScreen.h>
#include "LibTFT2p8.h"
#include "SimpleFont.h"

TouchScreen ts = TouchScreen(XP, YP, XM, YM);

typedef struct {int Xmin; int Ymin; int Xmax; int Ymax; void(*CallBack)(void);} TouchArea; 
TouchArea TouchAreas[NUMBEROFTOUCHAREAS];
byte TouchIndex;

//******************************************************************************************************
void tft_ClearScreen (void)
{
  Tft.fillScreen();
  tft_ClearTouchAreas();
}

//******************************************************************************************************
void tft_DrawLine (int X1, int Y1, int X2, int Y2, int Color)
{
#ifdef ROTATE  
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawLine (240-Y1, 320-X1, 240-Y2, 320-X2, Color);
    #else
        Tft.drawLine (Y1, 320-X1, Y2, 320-X2, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawLine (240-Y1, X1, 240-Y2, X2, Color);
    #else
        Tft.drawLine (Y1, X1, Y2, X2, Color);
    #endif
  #endif
#else
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawLine (240-X1, 320-Y1, 240-X2, 320-Y2, Color);
    #else
        Tft.drawLine (X1, 320-Y1, X2, 320-Y2, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawLine (240-X1, Y1, 240-X2, Y2, Color);
    #else
        Tft.drawLine (X1, Y1, X2, Y2, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawThickHorizontalLine (int X, int Y, int Length, int Thick, int Color)
{
  // Thickens to the positive Y direction
  for (int n = 0; n < Thick; n++)
#ifdef ROTATE  
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawVerticalLine (240-Y+n, 320-X-Length, Length, Color);
    #else
        Tft.drawVerticalLine (Y+n, 320-X-Length, Length, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawVerticalLine (240-Y-n, X, Length, Color);
    #else
        Tft.drawVerticalLine (Y+n, X, Length, Color);
    #endif
  #endif
#else
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawHorizontalLine (240-X-Length , 320-Y-n, Length, Color);
    #else
        Tft.drawHorizontalLine (X , 320-Y-n, Length, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawHorizontalLine (240-X-Length, Y+n, Length, Color);
    #else
        Tft.drawHorizontalLine (X , Y+n, Length, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawThickVerticalLine (int X, int Y, int Length, int Thick, int Color)
{
  // Thickens to the positive X direction
  for (int n = 0; n < Thick; n++)
#ifdef ROTATE  
  #ifdef FLIPUPDOWN  
    #ifdef FLIPLEFTRIGHT
        Tft.drawHorizontalLine (240-Y-Length, 320-X-n, Length, Color);
    #else
        Tft.drawHorizontalLine (Y, 320-X-n, Length, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawHorizontalLine (240-Y-Length, X+n, Length, Color);
    #else
        Tft.drawHorizontalLine (Y, X+n, Length, Color);
    #endif
  #endif
#else  
  #ifdef FLIPUPDOWN  
    #ifdef FLIPLEFTRIGHT
        Tft.drawVerticalLine (240-X-n, 320-Y-Length, Length, Color);
    #else
        Tft.drawVerticalLine (X+n, 320-Y-Length, Length, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawVerticalLine (240-X-n, Y, Length, Color);
    #else
        Tft.drawVerticalLine (X+n, Y, Length, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawThickRectangle (int X, int Y, int Width, int Height, int Thick, int Color)
{
  // Thickens to the inside
  for (int n = 0; n < Thick; n++)
#ifdef ROTATE
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.drawRectangle (240-Y-Width+n, 320-X-Height+n, Width-2*n, Height-2*n, Color);
    #else
      Tft.drawRectangle (Y+n, 320-X-Height+n, Width-2*n, Height-2*n, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.drawRectangle (240-Y-Width+n, X+n, Width-2*n, Height-2*n, Color);
    #else
      Tft.drawRectangle (Y+n, X+n, Width-2*n, Height-2*n, Color);
    #endif
  #endif
#else 
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.drawRectangle (240-X-Width+n, 320-Y-Height+n, Width-2*n, Height-2*n, Color);
    #else
      Tft.drawRectangle (X+n, 320-Y-Height+n, Width-2*n, Height-2*n, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.drawRectangle (240-X-Width+n, Y+n, Width-2*n, Height-2*n, Color);
    #else
      Tft.drawRectangle (X+n, Y+n, Width-2*n, Height-2*n, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawTriangle (int X1, int Y1, int X2, int Y2, int X3, int Y3, int Color)
{
tft_DrawLine (X1, Y1, X2, Y2, Color);
tft_DrawLine (X2, Y2, X3, Y3, Color);
tft_DrawLine (X1, Y1, X3, Y3, Color);
}

//******************************************************************************************************
void tft_DrawThickCircle (int X, int Y, int Diameter, int Thick, int Color)
{
  // Thickens to the inside
  for (int n = 0; n < Thick; n++)
#ifdef ROTATE
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.drawCircle (240-Y, 320-X, Diameter-n, Color);
    #else
      Tft.drawCircle (Y, 320-X, Diameter-n, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.drawCircle (240-Y, X, Diameter-n, Color);
    #else
      Tft.drawCircle (Y, X, Diameter-n, Color);
    #endif
  #endif
#else 
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.drawCircle (240-X, 320-Y, Diameter-n, Color);
    #else
      Tft.drawCircle (X, 320-Y, Diameter-n, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.drawCircle (240-X, Y, Diameter-n, Color);
    #else
      Tft.drawCircle (X, Y, Diameter-n, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawFilledCircle (int X, int Y, int Diameter, int Color)
{
#ifdef ROTATE
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.fillCircle (240-Y, 320-X, Diameter, Color);
    #else
      Tft.fillCircle (Y, 320-X, Diameter, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.fillCircle (240-Y, X, Diameter, Color);
    #else
      Tft.fillCircle (Y, X, Diameter, Color);
    #endif
  #endif
#else 
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
      Tft.fillCircle (240-X, 320-Y, Diameter, Color);
    #else
      Tft.fillCircle (X, 320-Y, Diameter, Color);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
      Tft.fillCircle (240-X, Y, Diameter, Color);
    #else
      Tft.fillCircle (X, Y, Diameter, Color);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawChar (char CH, int X, int Y, int Size, int fgColor, int bgColor)
{
byte Size8 = Size*8; 
#ifdef ROTATE  
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawChar (CH, 240-Y-Size8, 320-X-Size8, Size, fgColor, bgColor);
    #else
        Tft.drawChar (CH, Y, 320-X-Size8, Size, fgColor, bgColor);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawChar (CH, Y+(Size+2)*8, X, Size, fgColor, bgColor);
    #else
        Tft.drawChar (CH, Y, X, Size, fgColor, bgColor);
    #endif
  #endif
#else
  #ifdef FLIPUPDOWN
    #ifdef FLIPLEFTRIGHT
        Tft.drawChar (CH, 240-X-Size8, 320-Y-Size8, Size, fgColor, bgColor);
    #else
        Tft.drawChar (CH, X, 320-Y-Size8, Size, fgColor, bgColor);
    #endif
  #else
    #ifdef FLIPLEFTRIGHT
        Tft.drawChar (CH, 240-X-Size8, Y, Size, fgColor, bgColor);
    #else
        Tft.drawChar (CH, X, Y, Size, fgColor, bgColor);
    #endif
  #endif
#endif
}

//******************************************************************************************************
void tft_DrawString (char *S, int X, int Y, int Size, int fgColor, int bgColor)
{
while(*S)
  {
    tft_DrawChar(*S, X,Y, Size, fgColor, bgColor);
    *S++;
    X += 8*Size;   // Move cursor right/
#ifdef ROTATE    
    if(X >= 320-8*Size) return;
#else
    if(X >= 240-8*Size) return;
#endif
  }  
}

//******************************************************************************************************
void tft_ClearTouchAreas (void)
{
  for (int n=0; n<NUMBEROFTOUCHAREAS; n++)
  {
    TouchAreas[n].Xmin = 0;
    TouchAreas[n].Xmax = 0;
    TouchAreas[n].Ymin = 0;
    TouchAreas[n].Ymax = 0;
    TouchAreas[n].CallBack = 0;
    TouchIndex = 1;
  }  
}

//******************************************************************************************************
void tft_StoreTouchArea (int Xmin, int Ymin, int Xmax, int Ymax, void(*CallBackPtr)(void))
{
  if (TouchIndex >= NUMBEROFTOUCHAREAS) return; // safety measure
  TouchAreas[TouchIndex].CallBack = CallBackPtr;   
#ifdef ROTATE  
  #ifdef FLIPLEFTRIGHT 
    TouchAreas[TouchIndex].Xmin = 240-Ymax;
    TouchAreas[TouchIndex].Xmax = 240-Ymin;
  #else
    TouchAreas[TouchIndex].Xmin = Ymin;
    TouchAreas[TouchIndex].Xmax = Ymax;
  #endif
  #ifdef FLIPUPDOWN  
    TouchAreas[TouchIndex].Ymin = 320-Xmax;
    TouchAreas[TouchIndex].Ymax = 320-Xmin; 
  #else  
    TouchAreas[TouchIndex].Ymin = Xmin;
    TouchAreas[TouchIndex].Ymax = Xmax; 
  #endif
#else  
  #ifdef FLIPLEFTRIGHT 
    TouchAreas[TouchIndex].Xmin = 240-Xmax;
    TouchAreas[TouchIndex].Xmax = 240-Xmin;
  #else
    TouchAreas[TouchIndex].Xmin = Xmin;
    TouchAreas[TouchIndex].Xmax = Xmax;
  #endif
  #ifdef FLIPUPDOWN  
    TouchAreas[TouchIndex].Ymin = 320-Ymax;
    TouchAreas[TouchIndex].Ymax = 320-Ymin; 
  #else  
    TouchAreas[TouchIndex].Ymin = Ymin;
    TouchAreas[TouchIndex].Ymax = Ymax; 
  #endif
#endif
TouchIndex++;
}

//******************************************************************************************************
int tft_DetectTouch (void)
{
  // a point object holds x y and z coordinates
  Point p = ts.getPoint();
  p.x = map(p.x, TS_MINX, TS_MAXX, 0, 240);
  p.y = map(p.y, TS_MINY, TS_MAXY, 0, 320);
  
  // we have some minimum pressure we consider 'valid'
  // pressure of 0 means no pressing!
  if (p.z > __PRESURE)
  {
    for (int n=0; n<NUMBEROFTOUCHAREAS; n++)
    {
      if ((p.x > TouchAreas[n].Xmin) && (p.x < TouchAreas[n].Xmax)
          && (p.y > TouchAreas[n].Ymin) && (p.y < TouchAreas[n].Ymax))
      { 
        if (TouchAreas[n].CallBack != 0) TouchAreas[n].CallBack();
        return (n);  
      }
    }
    return (0);  // Any touch outside TouchAreas detected
  }
  return (-1);  // No touch detected
}

//******************************************************************************************************
void tft_init_LibTFT2p8 (void)
{
  Tft.TFTinit();
}

